@@ -50,7 +50,7 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
             **kwargs: Field values to set on the instance
         """
         super().__init__()
-        self._state_manager.set("dirty_fields", set())
+        self._state_manager.clear_dirty_fields()
 
         # Set history initialization flag before setting values
         if hasattr(self, "_history_initialized"):
@@ -79,24 +79,41 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
         pass
 
     def _get_all_data(self) -> dict:
-        """Get all field data.
+        """Get all field data, excluding deferred field proxies.
 
         Returns:
             Dictionary mapping field names to their current values
         """
-        return {name: getattr(self, name, None) for name in self._get_field_names()}
+        from .fields.proxies import DeferredFieldProxy, RelationFieldProxy
+
+        data = {}
+        for name in self._get_field_names():
+            value = getattr(self, name, None)
+            # Skip proxy objects to avoid serialization issues
+            if not isinstance(value, (DeferredFieldProxy, RelationFieldProxy)):
+                data[name] = value
+        return data
 
     def _get_dirty_data(self) -> dict:
-        """Get modified field data.
+        """Get modified field data, excluding deferred field proxies.
 
         Returns:
             Dictionary mapping dirty field names to their current values,
             or all field data if no dirty fields are tracked
         """
-        dirty_fields = self._state_manager.get("dirty_fields", set())
+        from .fields.proxies import DeferredFieldProxy, RelationFieldProxy
+
+        dirty_fields = self._state_manager.get_dirty_fields()
         if not dirty_fields:
             return self._get_all_data()
-        return {name: getattr(self, name, None) for name in dirty_fields}
+
+        data = {}
+        for name in dirty_fields:
+            value = getattr(self, name, None)
+            # Skip proxy objects to avoid serialization issues
+            if not isinstance(value, (DeferredFieldProxy, RelationFieldProxy)):
+                data[name] = value
+        return data
 
     def _set_primary_key_values(self, pk_values):
         """Set primary key values.
@@ -173,9 +190,7 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
                 if result.inserted_primary_key:
                     self._set_primary_key_values(result.inserted_primary_key)
                 # Clear dirty fields after successful save
-                dirty_fields = self._state_manager.get("dirty_fields", set())
-                if isinstance(dirty_fields, set):
-                    dirty_fields.clear()
+                self._state_manager.clear_dirty_fields()
                 return self
             except Exception as e:
                 raise PrimaryKeyError(f"Upsert operation failed: {e}") from e
@@ -201,14 +216,12 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
             raise PrimaryKeyError(f"Save operation failed: {e}") from e
 
         # Clear dirty fields after successful save
-        dirty_fields = self._state_manager.get("dirty_fields", set())
-        if isinstance(dirty_fields, set):
-            dirty_fields.clear()
+        self._state_manager.clear_dirty_fields()
         return self
 
     @emit_signals(Operation.SAVE)
     async def save(self, validate: bool = True, cascade: bool | None = None, session=None):
-        """Optimized save operation with cascade support and better error handling.
+        """Save operation with cascade support and automatic operation detection.
 
         Automatically determines whether to INSERT or UPDATE based on
         primary key presence. Uses dirty field tracking for efficient
@@ -230,152 +243,18 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
         if session is None:
             session = self.get_session()
 
-        # Check if we need to process cascade relationships
-        needs_cascade = (
-            self._state_manager.get("needs_cascade_save", False) if hasattr(self, "_state_manager") else False
-        )
-
-        if needs_cascade and cascade is not False:
-            await self._process_cascade_relationships(session)
-
-        # Determine if cascade should be used - only auto-detect if None
+        # Determine if cascade should be used
         if cascade is None:
             cascade = self._has_cascade_relations()
 
-        # Use cascade executor only if explicitly requested or auto-detected
+        # Use cascade executor for cascade operations
         if cascade:
             executor = CascadeExecutor()
-            await executor.execute_cascade_operation(self, "save", session)  # type: ignore[reportArgumentType]
-            return self
+            return await executor.execute_save_operation(self, validate=validate, session=session)  # type: ignore[reportArgumentType]
 
-        # Standard save operation using internal method
+        # Direct save operation without cascade
         return await self._save_internal(validate=validate, session=session)
 
-    async def _process_cascade_relationships(self, session):
-        """Process cascade relationships for this instance."""
-        if not hasattr(self, "_state_manager"):
-            return
-
-        cascade_relationships = self._state_manager.get("cascade_relationships", {})
-        if not cascade_relationships:
-            return
-
-        # Save self first to get primary key
-        await self._save_internal(session=session)
-
-        # Process each relationship with full update logic
-        for rel_name, new_related_objects in cascade_relationships.items():
-            await self._process_relationship_update(rel_name, new_related_objects, session)
-
-        # keep cascade_relationships
-        # self._state_manager.set("cascade_relationships", {})
-        self._state_manager.set("needs_cascade_save", False)
-
-    async def _process_relationship_update(self, rel_name: str, new_related_objects, session):
-        """Process complete relationship update: add, remove, modify."""
-        from .cascade import ForeignKeyInferrer
-
-        # Get relationship configuration
-        relationships = getattr(self.__class__, "_relationships", {})
-        if rel_name not in relationships:
-            return
-
-        rel_descriptor = relationships[rel_name]
-        if not (hasattr(rel_descriptor, "property") and hasattr(rel_descriptor.property, "cascade")):
-            return
-
-        cascade_str = rel_descriptor.property.cascade or ""
-        has_delete_orphan = "delete-orphan" in cascade_str
-
-        # Get current related objects from database
-        current_objects = await self._fetch_current_related_objects(rel_name, session)
-
-        # Convert to lists for processing
-        if new_related_objects is None:
-            new_objects = []
-        elif isinstance(new_related_objects, list):
-            new_objects = new_related_objects
-        else:
-            new_objects = [new_related_objects]
-
-        # Process updates
-        await self._update_relationship_objects(current_objects, new_objects, has_delete_orphan, session)
-
-        # Set foreign keys for new/updated objects
-        for obj in new_objects:
-            if hasattr(obj, "save"):
-                ForeignKeyInferrer.set_foreign_key(self, obj)
-                await obj.using(session).save(cascade=False)
-
-    async def _fetch_current_related_objects(self, rel_name: str, session) -> list:
-        """Fetch current related objects from database."""
-        relationships = getattr(self.__class__, "_relationships", {})
-        if rel_name not in relationships:
-            return []
-
-        rel_descriptor = relationships[rel_name]
-        if not hasattr(rel_descriptor.property, "resolved_model") or not rel_descriptor.property.resolved_model:
-            return []
-
-        related_model = rel_descriptor.property.resolved_model
-        foreign_keys = rel_descriptor.property.foreign_keys
-
-        if not foreign_keys:
-            return []
-
-        # fetch foreign keys
-        fk_field = foreign_keys if isinstance(foreign_keys, str) else foreign_keys[0]
-
-        # get pk
-        pk_value = getattr(self, self._get_primary_key_field())
-        if pk_value is None:
-            return []
-
-        current_objects = (
-            await related_model.objects.using(session).filter(getattr(related_model, fk_field) == pk_value).all()
-        )
-
-        return current_objects
-
-    async def _update_relationship_objects(
-        self, current_objects: list, new_objects: list, has_delete_orphan: bool, session
-    ):
-        """Update relationship objects: handle add, remove, modify."""
-        # Create ID mappings
-        current_by_id = {getattr(obj, "id", None): obj for obj in current_objects if getattr(obj, "id", None)}
-        new_by_id = {getattr(obj, "id", None): obj for obj in new_objects if getattr(obj, "id", None)}
-
-        # Find objects to remove (orphans)
-        if has_delete_orphan:
-            for obj_id, obj in current_by_id.items():
-                if obj_id and obj_id not in new_by_id:
-                    # This object is no longer in the relationship - delete it
-                    await obj.using(session).delete(cascade=False)
-
-        # Process existing objects for updates
-        for obj in new_objects:
-            obj_id = getattr(obj, "id", None)
-            if obj_id and obj_id in current_by_id:
-                # This is an existing object - check if it needs updating
-                current_obj = current_by_id[obj_id]
-                if self._object_has_changes(obj, current_obj):
-                    # Object has changes - it will be saved in the main loop
-                    pass
-
-    @staticmethod
-    def _object_has_changes(new_obj, current_obj) -> bool:
-        """Check if object has changes by comparing field values."""
-        # Simple implementation - compare key fields
-        field_names = getattr(new_obj, "_get_field_names", lambda: [])() or []
-        for field_name in field_names:
-            if field_name.startswith("_"):
-                continue
-            new_value = getattr(new_obj, field_name, None)
-            current_value = getattr(current_obj, field_name, None)
-            if new_value != current_value:
-                return True
-        return False
-
     @emit_signals(Operation.DELETE)
     async def delete(self, cascade: bool = True):
         """Delete this model instance from the database with cascade support.
@@ -389,16 +268,23 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
         if not self._has_primary_key_values():
             raise PrimaryKeyError("Cannot delete instance without primary key values")
 
-        # Use cascade executor if cascade is enabled
+        session = self.get_session()
+
+        # Use cascade executor for cascade operations
         if cascade:
             executor = CascadeExecutor()
-            await executor.execute_cascade_operation(self, "delete")  # type: ignore[reportArgumentType]
+            await executor.execute_delete_operation(self, session=session)
             return
 
-        # Standard delete operation without cascade
-        session = self.get_session()
-        table = self.get_table()
+        # Direct delete operation without cascade
+        await self._delete_internal(session=session)
 
+    async def _delete_internal(self, session=None):
+        """Internal delete operation without cascade handling."""
+        if session is None:
+            session = self.get_session()
+
+        table = self.get_table()
         try:
             pk_conditions = self._build_pk_conditions()
             stmt = delete(table).where(and_(*pk_conditions))
@@ -441,18 +327,16 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
         fresh_data = result.first()
 
         if fresh_data:
-            loaded_deferred_fields = self._state_manager.get("loaded_deferred_fields", set())
-            if isinstance(loaded_deferred_fields, set):
-                if fields:
-                    for i, field in enumerate(fields):
-                        setattr(self, field, fresh_data[i])
-                        if field in self._deferred_fields:
-                            loaded_deferred_fields.add(field)
-                else:
-                    for col_name, value in fresh_data._mapping.items():  # noqa
-                        setattr(self, col_name, value)
-                        if col_name in self._deferred_fields:
-                            loaded_deferred_fields.add(col_name)
+            if fields:
+                for i, field in enumerate(fields):
+                    setattr(self, field, fresh_data[i])
+                    if field in self._deferred_fields:
+                        self._state_manager.add_loaded_deferred_field(field)
+            else:
+                for col_name, value in fresh_data._mapping.items():  # noqa
+                    setattr(self, col_name, value)
+                    if col_name in self._deferred_fields:
+                        self._state_manager.add_loaded_deferred_field(col_name)
 
         return self
 
@@ -460,8 +344,15 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
         """Check if this model has any relationships configured for cascade operations.
 
         Returns:
-            True if any relationship has cascade=True, False otherwise
+            True if any relationship has cascade operations, False otherwise
         """
+        # Check for cascade relationships in state manager
+        if hasattr(self, "_state_manager"):
+            cascade_relationships = self._state_manager.get_cascade_relationships()
+            if cascade_relationships:
+                return True
+
+        # Check for configured cascade relationships
         relationships = getattr(self.__class__, "_relationships", {})
         for rel_descriptor in relationships.values():
             if hasattr(rel_descriptor, "property") and rel_descriptor.property.cascade:
@@ -521,9 +412,7 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
         if hasattr(self, "_get_relationship_fields") and name in self._get_relationship_fields():
             self._handle_relationship_assignment(name, value)
         elif not name.startswith("_") and hasattr(self, "_state_manager"):
-            dirty_fields = self._state_manager.get("dirty_fields", set())
-            if isinstance(dirty_fields, set):
-                dirty_fields.add(name)
+            self._state_manager.add_dirty_field(name)
         super().__setattr__(name, value)
 
     def _handle_relationship_assignment(self, field_name: str, value):
@@ -531,13 +420,9 @@ class ModelMixin(FieldCacheMixin, SignalMixin):
         if not hasattr(self, "_state_manager"):
             return
 
-        # Store relationship objects
-        cascade_relationships = self._state_manager.get("cascade_relationships", {})
-        cascade_relationships[field_name] = value if isinstance(value, list) else [value]  # type: ignore[reportOptionalSubscript]
-        self._state_manager.set("cascade_relationships", cascade_relationships)
-
-        # Mark for cascade save
-        self._state_manager.set("needs_cascade_save", True)
+        # Store relationship objects and mark for cascade save
+        relationship_value = value if isinstance(value, list) else [value] if value is not None else []
+        self._state_manager.set_cascade_relationship(field_name, relationship_value)
 
     def _get_relationship_fields(self) -> set[str]:
         """Get relationship field names from model metadata."""
